@@ -8,7 +8,9 @@ use log::warn;
 use proc_macro2::{Ident, Punct, Spacing, Span, TokenStream};
 use quote::{quote, ToTokens};
 
-use crate::util::{self, Config, ToSanitizedSnakeCase, ToSanitizedUpperCase, U32Ext};
+use crate::util::{
+    self, Config, ToSanitizedPascalCase, ToSanitizedSnakeCase, ToSanitizedUpperCase, U32Ext,
+};
 use anyhow::{anyhow, Result};
 
 pub fn render(
@@ -478,7 +480,7 @@ pub fn fields(
                     derive_from_base(mod_items, &base, &name_pc_r, &base_pc_r, &readerdoc);
                 } else {
                     let has_reserved_variant = evs.values.len() != (1 << width);
-                    let variants = Variant::from_enumerated_values(evs)?;
+                    let variants = Variant::from_enumerated_values(evs, config.pascal_enum_values)?;
                     let mut enum_items = TokenStream::new();
 
                     if variants.is_empty() {
@@ -557,7 +559,7 @@ pub fn fields(
 
                     mod_items.extend(quote! {
                         #[doc = #readerdoc]
-                        pub struct #name_pc_r(crate::FieldReader<#fty, #name_pc_a>);
+                        pub struct #name_pc_r(crate::FieldReader<#fty>);
 
                         impl #name_pc_r {
                             #[inline(always)]
@@ -568,7 +570,7 @@ pub fn fields(
                         }
 
                         impl core::ops::Deref for #name_pc_r {
-                            type Target = crate::FieldReader<#fty, #name_pc_a>;
+                            type Target = crate::FieldReader<#fty>;
 
                             #[inline(always)]
                             fn deref(&self) -> &Self::Target {
@@ -580,7 +582,7 @@ pub fn fields(
             } else {
                 mod_items.extend(quote! {
                     #[doc = #readerdoc]
-                    pub struct #name_pc_r(crate::FieldReader<#fty, #fty>);
+                    pub struct #name_pc_r(crate::FieldReader<#fty>);
 
                     impl #name_pc_r {
                         #[inline(always)]
@@ -590,7 +592,7 @@ pub fn fields(
                     }
 
                     impl core::ops::Deref for #name_pc_r {
-                        type Target = crate::FieldReader<#fty, #fty>;
+                        type Target = crate::FieldReader<#fty>;
 
                         #[inline(always)]
                         fn deref(&self) -> &Self::Target {
@@ -604,13 +606,12 @@ pub fn fields(
         if can_write {
             let new_pc_aw = Ident::new(&(name_pc.clone() + "_AW"), span);
             let name_pc_w = Ident::new(&(name_pc.clone() + "_W"), span);
-            let name_pc_cgw = Ident::new(&(name_pc.clone() + "_CGW"), span);
 
             let mut proxy_items = TokenStream::new();
             let mut unsafety = unsafety(f.write_constraint.as_ref(), width);
 
             if let Some((evs, base)) = lookup_filter(&lookup_results, Usage::Write) {
-                let variants = Variant::from_enumerated_values(evs)?;
+                let variants = Variant::from_enumerated_values(evs, config.pascal_enum_values)?;
 
                 if variants.len() == 1 << width {
                     unsafety = None;
@@ -750,27 +751,27 @@ pub fn fields(
             };
 
             if field_dim.is_some() {
-                mod_items.extend(quote! {
-                    #[doc = #doc]
-                    pub struct #name_pc_w<'a> {
-                        w: &'a mut W,
-                        offset: usize,
-                    }
+                if !config.const_generic {
+                    mod_items.extend(quote! {
+                        #[doc = #doc]
+                        pub struct #name_pc_w<'a> {
+                            w: &'a mut W,
+                            offset: usize,
+                        }
 
-                    impl<'a> #name_pc_w<'a> {
-                        #proxy_items
-                        #proxy_items_fa
-                    }
-                });
-
-                if config.const_generic {
+                        impl<'a> #name_pc_w<'a> {
+                            #proxy_items
+                            #proxy_items_fa
+                        }
+                    });
+                } else {
                     mod_items.extend(quote! {
                         #[doc = #cgdoc]
-                        pub struct #name_pc_cgw<'a, const O: usize> {
+                        pub struct #name_pc_w<'a, const O: usize> {
                             w: &'a mut W,
                         }
 
-                        impl<'a, const O: usize> #name_pc_cgw<'a, O> {
+                        impl<'a, const O: usize> #name_pc_w<'a, O> {
                             #proxy_items
                             #proxy_items_cg
                         }
@@ -790,15 +791,25 @@ pub fn fields(
             }
 
             if let Some((first, dim, increment, suffixes, suffixes_str)) = &field_dim {
-                let offset_calc = calculate_offset(*first, *increment, offset, false);
                 let doc = &util::replace_suffix(&description, suffixes_str);
-                w_impl_items.extend(quote! {
-                    #[doc = #doc]
-                    #inline
-                    pub unsafe fn #name_sc(&mut self, n: usize) -> #name_pc_w {
-                        #name_pc_w { w: self, offset: #offset_calc }
-                    }
-                });
+                if !config.const_generic {
+                    let offset_calc = calculate_offset(*first, *increment, offset, false);
+                    w_impl_items.extend(quote! {
+                        #[doc = #doc]
+                        #inline
+                        pub unsafe fn #name_sc(&mut self, n: usize) -> #name_pc_w {
+                            #name_pc_w { w: self, offset: #offset_calc }
+                        }
+                    });
+                } else {
+                    w_impl_items.extend(quote! {
+                        #[doc = #doc]
+                        #inline
+                        pub unsafe fn #name_sc<const O: usize>(&mut self) -> #name_pc_w<O> {
+                            #name_pc_w { w: self }
+                        }
+                    });
+                }
                 for (i, suffix) in (0..*dim).zip(suffixes.iter()) {
                     let sub_offset = offset + (i as u64) * (*increment as u64);
                     let name_sc_n = Ident::new(
@@ -822,8 +833,8 @@ pub fn fields(
                         w_impl_items.extend(quote! {
                             #[doc = #doc]
                             #inline
-                            pub fn #name_sc_n(&mut self) -> #name_pc_cgw<#sub_offset> {
-                                #name_pc_cgw { w: self }
+                            pub fn #name_sc_n(&mut self) -> #name_pc_w<#sub_offset> {
+                                #name_pc_w { w: self }
                             }
                         });
                     }
@@ -872,7 +883,7 @@ struct Variant {
 }
 
 impl Variant {
-    fn from_enumerated_values(evs: &EnumeratedValues) -> Result<Vec<Self>> {
+    fn from_enumerated_values(evs: &EnumeratedValues, pc: bool) -> Result<Vec<Self>> {
         let span = Span::call_site();
         evs.values
             .iter()
@@ -891,7 +902,14 @@ impl Variant {
                         .description
                         .clone()
                         .unwrap_or_else(|| format!("`{:b}`", value)),
-                    pc: Ident::new(&ev.name.to_sanitized_upper_case(), span),
+                    pc: Ident::new(
+                        &(if pc {
+                            ev.name.to_sanitized_pascal_case()
+                        } else {
+                            ev.name.to_sanitized_upper_case()
+                        }),
+                        span,
+                    ),
                     nksc: Ident::new(&nksc, span),
                     sc: Ident::new(&sc, span),
                     value,
