@@ -7,6 +7,7 @@ use inflections::Inflect;
 use proc_macro2::{Ident, Literal, Span, TokenStream};
 use quote::{quote, ToTokens};
 use std::path::{Path, PathBuf};
+use svd_rs::{MaybeArray, PeripheralInfo};
 
 use anyhow::{anyhow, bail, Context, Result};
 
@@ -26,6 +27,7 @@ pub struct Config {
     pub ignore_groups: bool,
     pub keep_list: bool,
     pub strict: bool,
+    pub pascal_enum_values: bool,
     pub output_dir: PathBuf,
     pub source_type: SourceType,
 }
@@ -41,6 +43,7 @@ impl Default for Config {
             ignore_groups: false,
             keep_list: false,
             strict: false,
+            pascal_enum_values: false,
             output_dir: PathBuf::from("."),
             source_type: SourceType::default(),
         }
@@ -219,10 +222,10 @@ pub fn escape_brackets(s: &str) -> String {
         })
 }
 
-pub fn name_of(register: &Register, ignore_group: bool) -> Cow<str> {
-    match register {
-        Register::Single(info) => info.fullname(ignore_group),
-        Register::Array(info, _) => replace_suffix(&info.fullname(ignore_group), "").into(),
+pub fn name_of<T: FullName>(maybe_array: &MaybeArray<T>, ignore_group: bool) -> Cow<str> {
+    match maybe_array {
+        MaybeArray::Single(info) => info.fullname(ignore_group),
+        MaybeArray::Array(info, _) => replace_suffix(&info.fullname(ignore_group), "").into(),
     }
 }
 
@@ -443,3 +446,9 @@ impl FullName for RegisterInfo {
         }
     }
 }
+
+impl FullName for PeripheralInfo {
+    fn fullname(&self, _ignore_group: bool) -> Cow<str> {
+        self.name.as_str().into()
+    }
+}
